package br.usp.ime.prolog.actions;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status; 

import org.eclipse.debug.core.model.IProcess;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;

import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import java.util.Iterator;
import java.util.ArrayList;

import br.usp.ime.prolog.PrologPlugin;
import br.usp.ime.prolog.launch.PrologConsole;
import br.usp.ime.prolog.preferences.IPrologPreferenceFields;


/**
 * @author Juliana 
 * 
 * Consults Prolog files into a Prolog session.
 *
 * This class is called by the Eclipse system when the menu 
 * 'Consult into Prolog Console' (triggered by a right-click event 
 *  in the Navigator view) is choosed, and a prolog file (*.pro, *.pl) 
 *  is selected in the Navigator view.
 */
public class ConsultPrologFile implements IObjectActionDelegate {
	private IFile fFile;
	private String fPrologRuntime;
	private ArrayList fCommands;
	private ArrayList fFilenames;
	private Process fInferiorProcess;
	private IProcess fEclipseProcHandle;


	/**
	 * Constructor for Action1.
	 */
	public ConsultPrologFile() {
		super();
		// Get the prolog console path and filename.
		fPrologRuntime = PrologPlugin.getDefault().getPreferenceStore()
								.getString(IPrologPreferenceFields.PREF_PROLOG_PATH);
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public final void setActivePart(final IAction action, final IWorkbenchPart targetPart) {
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public final void run(final IAction action) {
		// Activate a Prolog console and tell it 
		// to consult the selected files. 
		Shell shell = new Shell();
		
		try {
			fCommands = new ArrayList();
			Iterator it = fFilenames.iterator();
			
			// All the selected files will be consulted into
			// the Prolog session.
			while (it.hasNext()) {
				String filename = (String) it.next();
				fCommands.add(filename);
			}
			PrologConsole console = new PrologConsole(fCommands,null);
			
		} catch (CoreException e) {
			// Log the error to the Eclipse log.
			IStatus status = new Status(IStatus.ERROR,
			   PrologPlugin.getDefault().getDescriptor().getUniqueIdentifier(),0,
			   "Problems consulting prolog files into the interpreter.", e);
			PrologPlugin.log(status);	   
		}

		/* Teste
		MessageDialog.openInformation(shell,"Prolog Plug-in",fCommand);
		*/
	}

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 * 
	 * (This method is called when the selection in the workbench
	 *  changes).
	 */
	public final void selectionChanged(final IAction action, final ISelection selection)
	{	
		// Get the files selected in the workbench which triggered the
		// action.
		fFilenames = new ArrayList();
		if (selection != null && selection instanceof IStructuredSelection) {
			IStructuredSelection structured = (IStructuredSelection) selection;
			Iterator it = structured.iterator();
			
			while (it.hasNext()) {
				Object o = it.next();
				fFile = (IFile) o;
				fFilenames.add(fFile.getLocation().toString());				
			}
		} 
	}
}
